pub struct HashConsed<T>(HashByAddr<Arc<T>>);Expand description
Hash-consed data structure: a reference-counted wrapper that guarantees that two equal value will be stored at the same address. This makes it possible to use the pointer address as a hash value.
Tuple Fields
0: HashByAddr<Arc<T>>Implementations
Sourceimpl<T> HashConsed<T>where
T: HashConsable,
impl<T> HashConsed<T>where
T: HashConsable,
Sourcepub fn new(inner: T) -> Self
pub fn new(inner: T) -> Self
Deduplicate the values by hashing them. This deduplication is crucial for the hashing
function to be correct. This is the only function allowed to create Self values.
Sourcepub fn from_arc(inner: Arc<T>) -> Self
pub fn from_arc(inner: Arc<T>) -> Self
Rarely used: in case we already have an Arc, may avoid an allocation.
Sourcepub fn as_mut(&mut self) -> impl DerefMut<Target = T>
pub fn as_mut(&mut self) -> impl DerefMut<Target = T>
Get a reference to the pointed-to value that can be mutated. Avoids cloning/allocation if this is the sole pointer to that value.
The returned value must not be leaked as this would break the hash-consing invariant.
Sourcepub fn with_inner_mut<R>(&mut self, f: impl FnOnce(&mut T) -> R) -> R
pub fn with_inner_mut<R>(&mut self, f: impl FnOnce(&mut T) -> R) -> R
Clones if needed to get mutable access to the inner value.